/*  

    This file is part of the program Spherical  
    Copyright (C) 2007-2008  André Schulz

    Spherical is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Spherical is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Spherical; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "sphereCanvas.h"
#include <cmath>
#include <QtOpenGL>
#include <iostream>


double PI = 3.141592653589793238462643; 

GLfloat winkel(GLfloat x1,GLfloat y1,GLfloat z1,
	       GLfloat x2,GLfloat y2,GLfloat z2)

{
  GLfloat w=acos((x1*x2+y1*y2+z1*z2)/
    (sqrt(x1*x1+y1*y1+z1*z1)*sqrt(x2*x2+y2*y2+z2*z2)));
  
  if ((x1*y2-y1*x2)<0) w=w*-1;
  if (((x1*y2-y1*x2)==0) && (x1>x2)) w=w*-1;
  return w/(2*PI)*360;
}




void sphereCanvas::assignGraph(geomGraph* graph)

{
  g=graph;
  arcList = glGenLists(1);
}

void sphereCanvas::drawGraph()

{
   

  //  glCallList(arcList);

  const vertex *aktVertex;
  const edge   *aktEdge;
  
  //glNewList(arcList,GL_COMPILE);	

  glColor3f(0,0,0);
  aktEdge=g->getFirstEdge();
  while (aktEdge)  {  
    drawEdge(*aktEdge);
    aktEdge=g->getNextEdge();
  }

  // to increase speed we draw all vertices without calling drawVertex()
  // in red color
   
  glColor3f(0,0,0);	 
  glBegin(GL_POINTS);
  aktVertex=g->getFirstVertex();
  while (aktVertex)  {
           glVertex3f(aktVertex->getSX(),aktVertex->getSY(),aktVertex->getSZ());
	   aktVertex=g->getNextVertex();
  }
  glEnd();
 


}


void sphereCanvas::refreshArcs(int style)

{
  const vertex *aktVertex;
  const edge   *aktEdge;
  
  glNewList(arcList,GL_COMPILE);	

  glColor3f(0,0,0);
  aktEdge=g->getFirstEdge();
  while (aktEdge)  {  
    drawEdge(*aktEdge);
    aktEdge=g->getNextEdge();
  }

  // to increase speed we draw all vertices without calling drawVertex()
  // in red color
   
  glColor3f(0,0,0);	 
  glBegin(GL_POINTS);
  aktVertex=g->getFirstVertex();
  while (aktVertex)  {
           glVertex3f(aktVertex->getSX(),aktVertex->getSY(),aktVertex->getSZ());
	   aktVertex=g->getNextVertex();
  }
  glEnd();
 
  glEndList();
   
  
}

void sphereCanvas::drawVertex(const vertex v)

{
  glBegin(GL_POINTS);
  glVertex3f(v.getSX(),v.getSY(),v.getSZ());
  glEnd(); 
}


void sphereCanvas::drawEdge(const edge e)

{

 
  const vertex pointA=e.getA();
  const vertex pointB=e.getB();

  
  GLfloat x1=pointA.getSX();
  GLfloat y1=pointA.getSY();
  GLfloat z1=pointA.getSZ();

  GLfloat x2=pointB.getSX();
  GLfloat y2=pointB.getSY();
  GLfloat z2=pointB.getSZ();

  //  glColor3f(1,1,1);


  //degeneriert?

  if (x1==-x2 && y1==-y2 && z1==-z2) x2=x2+0.001;


  // Test Kreis durch punkte 

  GLUquadricObj *kreis;


  // Schnittgerade berechen 
  GLfloat xs=x1*z2-x2*z1;
  GLfloat ys=z2*y1-y2*z1;
  GLfloat zs=0;

 
  // Schnittwinkel berechnen
  GLbyte orientation=1;
  if (winkel(x1,y1,z1,x2,y2,z2)<0) orientation=-1;

  GLfloat angle;
  angle=winkel(0,0,1,xs,ys,-(x1*y2-y1*x2));
  if (angle>0) {
    xs=-xs,ys=-ys;};

  
  kreis= gluNewQuadric();
  gluQuadricDrawStyle(kreis,GLU_SILHOUETTE); 

  glPushMatrix();
  
  glRotatef(angle,xs,ys,zs);
  GLfloat winkelA2=(winkel(xs,ys,zs,x1,y1,z1));
  GLfloat winkel1=(winkel(0,1,0,xs,ys,zs));
  GLfloat winkelB2=(winkel(xs,ys,zs,x2,y2,z2));
  GLfloat winkelA=-winkel1+orientation*winkelA2;
  GLfloat winkelB=-winkel1+orientation*winkelB2;
  if (winkelB<0) winkelB=winkelB+360;
  GLfloat arcLength=(int)(winkelB-winkelA) % 360;
  if (orientation==-1)  arcLength=(int)(-360+arcLength) %360;
  if (abs(arcLength)<30)  gluPartialDisk(kreis,1,1,4,2,winkelA,arcLength);
  else  gluPartialDisk(kreis,1,1,abs(arcLength/6),2,winkelA,arcLength);

  gluDeleteQuadric(kreis);
  
  glPopMatrix();
  

}


void sphereCanvas::drawEdgeOriented(const edge e,int o)

{
  if (e.getOrientation()==o) drawEdge(e);
  else
    {
      vertex v1=e.getA();
      vertex v2=e.getB();
      edge e2;
      e2.set(v2,v1);
      drawEdge(e2);
    }

}
